----------------------------------------------------------------------------------
-- Company:		HiassofT
-- Engineer:		Matthias Reichl
-- 
-- Create Date:		2011-06-11
-- Module Name:		CartSPI
-- Project Name:	TheCart
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.NUMERIC_STD.ALL;

library work;
use work.all;
use CartDef.all;

entity CartSPI is
    Port (	clk_register: in std_logic;
		a: in vec12;
		d_in: in std_logic_vector;
		rw: in std_logic;
		reset_n: in std_logic;
		cctl: in std_logic;
		cs: out std_logic := '1';
		sck: out std_logic := '0';
		si: out std_logic;
		so: in std_logic;
		output: out data_output
	);
end CartSPI;

architecture Behavioral of CartSPI is

begin

	spi_write: process(clk_register, a, d_in, rw, reset_n, cctl)
	begin
		if falling_edge(clk_register) then
			if (reset_n = '0') then
				cs <= '1';
				si <= '1';
				sck <= '0';
			else
				if (rw = '0') and (cctl = '0') and (a(7 downto 0) = spi_adr) then
					si <= d_in(7);
					cs <= d_in(1);
					sck <= d_in(0);
				end if;
			end if;
		end if;
	end process spi_write;

	spi_read: process(a, cctl, rw, cctl, so)
	begin
		output.dout_enable <= false;
		output.dout <= (others => '0');
		if (rw = '1') and (cctl = '0') and (a(7 downto 0) = spi_adr) then
			output.dout_enable <= true;
			output.dout <= so & "0000000";
		end if;
	end process spi_read;

end Behavioral;

