----------------------------------------------------------------------------------
-- Company: mega-hz, HiassofT
-- Engineer: Wolfram Fischer, Matthias Reichl
-- 
-- Create Date:    13:43:05 11/01/2011 
-- Design Name: 
-- Module Name:    TheCart - Behavioral 
-- Project Name:   128MB CARTRIDGE fr ATARI XL/XE Computer
-- Target Devices: XILINX 95144XL
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library work;
use work.all;
use CartDef.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity TheCart is
    port(
        adr:  in std_logic_vector(12 downto 0);
        data: inout std_logic_vector(7 downto 0);
        rw: in std_logic;
        phi2: in std_logic;
        phi2short: in std_logic;
        reset_n: in std_logic;
        cctl: in std_logic;
        s4: in std_logic;
        s5: in std_logic;
        rd4: out std_logic;
        rd5: out std_logic;
 
        ram_rom_adr: out std_logic_vector(26 downto 0);
        ram_rom_data: inout std_logic_vector(7 downto 0);

        ram_rom_oe: out std_logic;
        ram_rom_we: out std_logic;

        ram_ce: out std_logic;
        rom_ce: out std_logic;

        rom_reset: out std_logic;

        eeprom_cs: out std_logic := '1';
        eeprom_sck: out std_logic := '0';
        eeprom_so: in std_logic;
        eeprom_si: out std_logic;
        mod_en: out std_logic

--        A15: in std_logic;
--        A14: in std_logic;
--        A13: in std_logic;
--        REFRESH:inout std_logic;
--        MMU_IN:in std_logic;
--        MMU_OUT:out std_logic;
--        EE_CS: out std_logic;
--        EE_SO: in std_logic;
--        EE_SI: out std_logic;
--        EE_SCK: out std_logic;
    );

    -- pin locations for XC95144XL (TQFP100)
    attribute LOC: String;

    -- Modulport
    attribute LOC of adr: signal is "P97 P14 P15 P96 P95 P94 P92 P90 P87 P86 P89 P91 P93";
    attribute LOC of data: signal is "P12 P13 P6 P1 P10 P8 P9 P11";
    attribute LOC of rw: signal is "P99";
    attribute LOC of phi2: signal is "P22";
    attribute LOC of phi2short: signal is "P23";
    attribute LOC of reset_n: signal is "P82";
    attribute LOC of cctl: signal is "P27";
    attribute LOC of s4: signal is "P3";
    attribute LOC of s5: signal is "P4";
    attribute LOC of rd4: signal is "P81";
    attribute LOC of rd5: signal is "P79";

    -- Flash + RAM
    attribute LOC of ram_rom_adr: signal is "P24 P29 P78 P77 P63 P64 P65 P61 P60 P28 P76 P74 P72 P71 P70 P68 P67 P66 P59 P58 P56 P55 P54 P53 P52 P43 P25";
    attribute LOC of ram_rom_data: signal is "P32 P33 P35 P36 P37 P39 P40 P41";

    attribute LOC of ram_rom_oe: signal is "P42";
    attribute LOC of ram_rom_we: signal is "P50";

    attribute LOC of ram_ce: signal is "P85";
    attribute LOC of rom_ce: signal is "P49";

    attribute LOC of rom_reset: signal is "P30";

    -- SPI eeprom
    attribute LOC of eeprom_cs: signal is "P16";
    attribute LOC of eeprom_so: signal is "P17";
    attribute LOC of eeprom_si: signal is "P20";
    attribute LOC of eeprom_sck: signal is "P18";

    attribute LOC of mod_en: signal is "P73";
    -- zustzliche Siganle bei internen Einbau

--    attribute LOC of A15: signal is "P2";
--    attribute LOC of A14: signal is "P7";
--    attribute LOC of A13: signal is "P19";
--    attribute LOC of REFRESH: signal is "P80";
--    attribute LOC of MMU_IN: signal is "P34";
--    attribute LOC of MMU_OUT: signal is "P46";

    -- fix warnings in Xilinx software
    attribute BUFG: String;
    attribute BUFG of phi2short: signal is "CLK";

    -- set the default output slew rate to "slow"
    attribute SLOW: String;
    attribute SLOW of TheCart: entity is "TRUE";

end TheCart;

architecture Behavioral of TheCart is

signal cart_mem: cart_mem_output;
signal cart_data: data_output;
signal spi_data: data_output;

signal write_enable_out: std_logic;

signal reset_n_sync1: std_logic := '0';
signal reset_n_sync: std_logic := '0';

signal powerup_n: std_logic := '0';

begin
	-- two-stage synchronizer for reset input
	sync_reset: process(phi2short)
	begin
		if falling_edge(phi2short) then
			reset_n_sync <= reset_n_sync1;
			reset_n_sync1 <= reset_n;
		end if;
	end process sync_reset;

	-- powerup signal: initialized to 0 by default,
	-- set to 1 once reset_n goes high
	powerup: process(phi2short)
	begin
		if falling_edge(phi2short) then
			if (reset_n_sync = '1') then
				powerup_n <= '1';
			end if;
		end if;
	end process powerup;

	use_cart_logic: entity CartLogic
	port map(
		clk_register => phi2short,
		a => adr,
		d_in => data,
		rw => rw,
		reset_n => reset_n_sync,
		cctl => cctl,
		s4 => s4,
		s5 => s5,
		mem_out => cart_mem,
		data_out => cart_data,
		write_enable_out => write_enable_out
	);

	use_cart_spi: entity CartSPI
	port map(
		clk_register => phi2short,
		a => adr,
		d_in => data,
		rw => rw,
		reset_n => reset_n,
		cctl => cctl,
		cs => eeprom_cs,
		sck => eeprom_sck,
		si => eeprom_si,
		so => eeprom_so,
		output => spi_data
	);


	set_ram_rom: process(cart_mem, cart_data, spi_data, phi2, phi2short, rw)
	begin
	    ram_rom_adr <= cart_mem.adr;
	    ram_rom_oe <= '1';
	    ram_rom_we <= '1';
	    ram_ce <= '1';
	    rom_ce <= '1';
	    rd4 <= '0';
	    rd5 <= '0';
	    data <= (others => 'Z');
	    ram_rom_data <= (others => 'Z');

	    if (rw = '1') then
		ram_rom_oe <= NOT phi2;
	    else
		ram_rom_we <= NOT phi2short;
	    end if;
	    
	    if cart_mem.ram_access then
		ram_ce <= '0';
	    end if;
	    
	    if cart_mem.rom_access then
		rom_ce <= '0';
	    end if;

	    if (rw = '1') then
		if (cart_data.dout_enable or spi_data.dout_enable) then
			data <= cart_data.dout or spi_data.dout;
		else
			if (cart_mem.ram_access or cart_mem.rom_access) then
				data <= ram_rom_data;
			end if;
		end if;
	    else
		ram_rom_data <= data;
	    end if;

	    if (cart_mem.rd4) then
	        rd4 <= '1';
	    end if;
	    if (cart_mem.rd5) then
	        rd5 <= '1';
	    end if;

	end process set_ram_rom;

	-- rom_reset <= '1';
	--rom_reset <= reset_n_sync;
	rom_reset <= powerup_n;
	
	mod_en <= not write_enable_out;

end Behavioral;

